/*
// author : Sven Goethel - Stapenhorststrasse 35a - 33615 Bielefeld
*/

#include "metatype.h"


/*   *******************************************************
           t _ i n t         structure
     *******************************************************  */
static rtti * RttiInt=NULL;

static void IntReset(void)
{
  RttiInt=NULL;
}

const rtti * const GetRttiInt       (void)
{
  if(RttiInt==NULL) INT_ERR(__LINE__);
  return RttiInt;
}

const rtti * const IntRegisterRtti  (void)
{
  if(RttiInt == NULL)
  {
    RttiInt = NewRttiType(sizeof(t_int));
    RegisterRttiFct(RttiInt,
                    (T_FCmpRtti) CmpInt,
                    (T_FCreateRtti) CreateInt,
                    (T_FCopyCreateRtti) CopyCreateInt,
                    (T_FDestroyRtti) DestroyInt,
                    (T_FResetRtti) IntReset
                   );
  }
  return GetRttiInt();
}

rtti * CreateInt     ( void )
{
  t_int * res = NULL;
  if(!RttiInt) INT_ERR(__LINE__);
  res = (t_int *)calloc(1,sizeof(t_int));
  if(!res) INT_ERR(__LINE__);
  res->rtype.type=RttiInt->type;
  res->rtype.sz=RttiInt->sz;
  return C_RTTI(res);
}

t_int * ICopyCreateInt ( int i )
{
  t_int * res = C_INT(CreateInt());
  res->i=i;
  return res;
}

rtti * CopyCreateInt ( const t_int *a )
{
  if(!a) INT_ERR(__LINE__);
  return C_RTTI(ICopyCreateInt(a->i));
}

void   DestroyInt    ( t_int *a )
{
  if(!a) INT_ERR(__LINE__);
  free(a);
}

int    CmpInt(const t_int *a, const t_int *b)
{ return a->i==b->i; }


/*   *******************************************************
           t _ r e a l         structure
     *******************************************************  */

static rtti * RttiReal=NULL;

static void RealReset(void)
{
  RttiReal=NULL;
}

const rtti * const GetRttiReal       (void)
{
  if(RttiReal==NULL) INT_ERR(__LINE__);
  return RttiReal;
}

const rtti * const RealRegisterRtti  (void)
{
  if(RttiReal == NULL)
  {
    RttiReal = NewRttiType(sizeof(t_real));
    RegisterRttiFct(RttiReal,
                    (T_FCmpRtti) CmpReal,
                    (T_FCreateRtti) CreateReal,
                    (T_FCopyCreateRtti) CopyCreateReal,
                    (T_FDestroyRtti) DestroyReal,
                    (T_FResetRtti) RealReset
                   );
  }
  return GetRttiReal();
}

rtti * CreateReal     ( void )
{
  t_real * res = NULL;
  if(!RttiReal) INT_ERR(__LINE__);
  res = (t_real *)calloc(1,sizeof(t_real));
  if(!res) INT_ERR(__LINE__);
  res->rtype.type=RttiReal->type;
  res->rtype.sz=RttiReal->sz;
  return C_RTTI(res);
}

t_real * DCopyCreateReal ( double d )
{
  t_real * res = C_REAL(CreateReal());
  res->r=d;
  return res;
}


rtti * CopyCreateReal ( const t_real *a )
{
  if(!a) INT_ERR(__LINE__);
  return C_RTTI(DCopyCreateReal (a->r));
}

void   DestroyReal    ( t_real *a )
{
  if(!a) INT_ERR(__LINE__);
  free(a);
}

int    CmpReal(const t_real *a, const t_real *b)
{ return a->r==b->r; }



/*   *******************************************************
           t _ c o o r d         structure
     *******************************************************  */

static rtti * RttiCoord=NULL;

static void CoordReset(void)
{
  RttiCoord=NULL;
}

const rtti * const GetRttiCoord       (void)
{
  if(RttiCoord==NULL) INT_ERR(__LINE__);
  return RttiCoord;
}

const rtti * const CoordRegisterRtti  (void)
{
  if(RttiCoord == NULL)
  {
    RttiCoord = NewRttiType(sizeof(t_coord));
    RegisterRttiFct(RttiCoord,
                    (T_FCmpRtti) Coord_IsEqual,
                    (T_FCreateRtti) CreateCoord,
                    (T_FCopyCreateRtti) CopyCreateCoord,
                    (T_FDestroyRtti) DestroyCoord,
                    (T_FResetRtti) CoordReset
                   );
  }
  return GetRttiCoord();
}

rtti * CreateCoord     ( void )
{
  t_coord * res = NULL;
  if(!RttiCoord) INT_ERR(__LINE__);
  res = (t_coord *)calloc(1,sizeof(t_coord));
  if(!res) INT_ERR(__LINE__);
  res->rtype=*RttiCoord;
  res->x.rtype=*RttiReal;
  res->y.rtype=*RttiReal;
  res->z.rtype=*RttiReal;
  return C_RTTI(res);
}

t_coord * RCopyCreateCoord ( const t_real *x,const t_real *y,const t_real *z)
{
  if(!x || !y || !z ) INT_ERR(__LINE__);
  return DCopyCreateCoord(x->r, y->r, z->r);
}

t_coord * DCopyCreateCoord ( double x, double y, double z)
{
  t_coord * res = C_COORD(CreateCoord());
  res->x.r=x;
  res->y.r=y;
  res->z.r=z;
  return res;
}

rtti * CopyCreateCoord ( const t_coord *a )
{
  if(!a) INT_ERR(__LINE__);
  return C_RTTI(RCopyCreateCoord(&(a->x), &(a->y), &(a->z)));
}

void   DestroyCoord    ( t_coord *a )
{
  if(!a) INT_ERR(__LINE__);
  free(a);
}

int Coord_IsEqual(const t_coord *a, const t_coord *b)
{ return a->x.r==b->x.r && a->y.r==b->y.r && a->z.r==b->z.r; }

void Coord_Length(t_real *result, const t_coord *pa)
{
  double r = sqrt( (pa->x).r * (pa->x).r  +
                   (pa->y).r * (pa->y).r  +
                   (pa->z).r * (pa->z).r
                 );
  result->rtype=*RttiReal;
  result->r=r;
}

void Coord_Sum(t_coord *result, const t_coord *pa, const t_coord *pb)
{ *result=*pa; result->x.r+=pb->x.r; result->y.r+=pb->y.r; result->z.r+=pb->z.r; }

void Coord_Sub(t_coord *result, const t_coord *pa, const t_coord *pb)
{ *result=*pa; result->x.r-=pb->x.r; result->y.r-=pb->y.r; 
  result->z.r-=pb->z.r;
}

void Coord_Mul(t_coord *result, const t_coord *pa, const t_real f)
{ *result=*pa; result->x.r*=f.r; result->y.r*=f.r; result->z.r*=f.r; }

void Coord_Div(t_coord *result, const t_coord *pa, const t_real f)
{ *result=*pa; if(f.r==0) INT_ERR(__LINE__);
	result->x.r/=f.r; result->y.r/=f.r; result->z.r/=f.r; 
}

void Coord_ScalarMul(t_real *result, const t_coord *pa, const t_coord *pb)
{
  result->rtype=*RttiReal;
  result->r= pa->x.r*pb->x.r+pa->y.r*pb->y.r+pa->z.r*pb->z.r;
}

