/*
// author : Sven Goethel - Stapenhorststrasse 35a - 33615 Bielefeld
*/

#ifndef _FXMTYPES_H

  #ifndef _METATYPES_H
    #define _METATYPES_H 

    #include <math.h>
    #include "rtti.h"
    #include "strclass.h"
    #include "bool.h"

    #define EPSILON    (0.1/1000)

    typedef unsigned char BYTE;

    typedef StrClass t_string;

    typedef bool t_bool;

    typedef enum { L, A } t_contour;

    typedef enum { CW, CCW } t_rotation;

    typedef char  t_char;

    typedef struct  t_int
    {
      rtti rtype;
      int i;
    } t_int;

    #define C_INT(a) ((t_int *)(a))

    const rtti * const GetRttiInt       (void);
    const rtti * const IntRegisterRtti  (void);
    rtti * CreateInt     ( void );
    rtti * CopyCreateInt ( const t_int * );
    t_int * ICopyCreateInt ( int i );
    void   DestroyInt    ( t_int * );
    int    CmpInt(const t_int *a, const t_int *b);

    typedef long  t_long;

    typedef struct t_real
    {
      rtti rtype;
      double r;
    } t_real;

    #define C_REAL(a) ((t_real *)(a))

    const rtti * const GetRttiReal       (void);
    const rtti * const RealRegisterRtti  (void);
    rtti * CreateReal     ( void );
    rtti * CopyCreateReal ( const t_real * );
    t_real * DCopyCreateReal (double d );
    void   DestroyReal    ( t_real * );
    int    CmpReal(const t_real *a, const t_real *b);


    #ifndef M_PI
      #define M_PI 3.141592653
    #endif

    typedef struct t_coord
    {
      rtti    rtype;
      t_real  x;
      t_real  y;
      t_real  z;
    } t_coord ;

    #define C_COORD(a) ((t_coord *)(a))

    const rtti * const GetRttiCoord       (void);
    const rtti * const CoordRegisterRtti  (void);

    rtti * CreateCoord     ( void );
    rtti * CopyCreateCoord ( const t_coord * );
    t_coord * RCopyCreateCoord ( const t_real *,const t_real *,const t_real *);
    t_coord * DCopyCreateCoord ( double, double, double );
    void   DestroyCoord    ( t_coord * );

    int Coord_IsEqual(const t_coord *a, const t_coord *b);

    void Coord_Length(t_real *result, const t_coord *pa) ;

    void Coord_Sum(t_coord *result, const t_coord *pa, const t_coord *pb);

    void Coord_Sub(t_coord *result, const t_coord *pa, const t_coord *pb);
    
    void Coord_Mul(t_coord *result, const t_coord *pa, const t_real f);

    void Coord_Div(t_coord *result, const t_coord *pa, const t_real f);
    
    void Coord_ScalarMul(t_real *result, const t_coord *pa, const t_coord *pb);
    
  #endif
#endif


