// author : Sven Goethel - Stapenhorststrasse 35a - 33615 Bielefeld
//
/*************************************************************************/
/*
 *  Datei           : metageom.cc
 *  Inhalt          : Definitionen fuer Klassen :
 *  Programmierer   : Sven Goethel
 *            Stapenhorststr.35a
 *            33615 Bielefeld
 *            0521/139228
 *
 *  $Log: metageom.cc $
 *  Revision 1.2  1995/04/07 16:54:54  Sven_Goethel_DFP
 *  *** empty log message ***
 *  Revision 1.1  1995/04/04 10:37:30  Sven_Goethel
 *  Initial revision
 */
/*************************************************************************/

#ifdef _Windows
  #include <afx.h>
#else
  #include <assert.h>
	#define TRACE(a)
#endif  
# include "metageom.h"

/*------------------------------------------------------------------------------

  ElementFunktion der Klasse MetaGeometry
  --------------------------------------------
  t_real ::AddWinkel(const t_real a, const t_real b)

  - Beschreibung
  --------------
  Addiert Winkel a und Winkel b

  - Parameter
  -----------
  E: a,b  - die Winkel-Summanden


  - allgemeine Ein- und Ausgabeeinheiten
  --------------------------------------
  keine

  - Returncode
  ------------
  0<=Winkel<=360 in Grad !!!!

  - Historie
  ----------
  29.6.1994 Sven Goethel

------------------------------------------------------------------------------*/
t_real MetaGeometry::AddWinkel(const t_real a, const t_real b)
{
  t_real result=a+b;
  if(result>360.0) 
    result-=360;
  return result;
}

/*------------------------------------------------------------------------------

  ElementFunktion der Klasse MetaGeometry
  --------------------------------------------
  t_real ::SubWinkel(const t_real a, const t_real b)

  - Beschreibung
  --------------
  Subtrahiert Winkel b von Winkel a  => a-b

  - Parameter
  -----------
  E: a,b  - die Winkel-Summanden


  - allgemeine Ein- und Ausgabeeinheiten
  --------------------------------------
  keine

  - Returncode
  ------------
  0<=Winkel<=360 in Grad !!!! ( a-b )

  - Historie
  ----------
  29.6.1994 Sven Goethel

------------------------------------------------------------------------------*/
t_real MetaGeometry::SubWinkel(const t_real a, const t_real b)
{
  t_real result=a-b;
  if(result.value()<0) 
    result+=360;
  return result;
}

/*------------------------------------------------------------------------------

  ElementFunktion der Klasse MetaGeometry
  --------------------------------------------
              t_real VektorBetrag(const t_coord &P0, const t_coord &PM,
                  const t_coord &P1, t_rotation rot)

  - Beschreibung
  --------------
  Berechnet den Betrag des Vektors von P1 nach P2.

  - Parameter
  -----------
  E: P1, P2 Die beiden t_coorde.

  - allgemeine Ein- und Ausgabeeinheiten
  --------------------------------------
  keine

  - Returncode
  ------------
  der Betrag

  - Historie
  ----------
  16.6.1994 Sven Goethel

------------------------------------------------------------------------------*/
t_real MetaGeometry::VektorBetrag(const t_coord &P0, const t_coord &PM,
                  const t_coord &P1, t_rotation rot)
{
  t_real Radius=VektorBetrag(PM,P0);
  // ... Da dieser innere Winkel genau anders Interpretiert wird,
  // wurde Start und Endpunkt umgedreht !!!
  t_real Gamma = GetWinkel12 ( P1, PM, P0, rot );
  return Gamma*Radius*M_PI/180;
}

t_real MetaGeometry::VektorBetrag(const t_coord &P1, const t_coord &P2)
{
  return  sqrt ( 
           abs(  
             ( P1.x-P2.x ) *  ( P1.x-P2.x )
             +  ( P1.y-P2.y ) *  ( P1.y-P2.y )
             +  ( P1.z-P2.z ) *  ( P1.z-P2.z )
            )
         ) ;
}

/*------------------------------------------------------------------------------

  ElementFunktion der Klasse Geometry
  --------------------------------------------
  double ::GetWinkelAbsolut(const FxmObject &FObj, Position Pos)

  - Beschreibung
  --------------
  Berechnet den Winkel Absolut zur Positiven X-Achse.


  - Parameter
  -----------
  E: FObj   Die Kontur-Objekte
  E: Pos      Anfang oder Ende

  - allgemeine Ein- und Ausgabeeinheiten
  --------------------------------------
  keine

  - Returncode
  ------------
  den Winkel (0-90(I.),90-180(II.),180-270(III.),270-360(IV.))
  zur X-Achse des I.Quadranten in GRAD....

  - Historie
  ----------
  29.6.1994 Sven Goethel

------------------------------------------------------------------------------*/
t_real MetaGeometry::GetWinkelAbsolut(const t_coord &Pp, const t_coord &PM,
                                      t_rotation Rot) 
{
  // BOGEN, Winkel der Tangente des Bogenendpunktes und der X-Achse bilden.

  // PM - Mittelpunkt
  // Pp - Winkelbildende Punkt

  // Winkel Beta zwischen positiver X-Achse (I.Quadrant)  und PM->Pp,  !
  t_real r=GetWinkelAbsolut(PM, Pp);
  if(Rot==CCW) r=AddWinkel(r, 90); 
  else r=SubWinkel(r, 90); 
  return r;
}

/*------------------------------------------------------------------------------

  ElementFunktion der Klasse MetaGeometry
  --------------------------------------------
  t_real ::GetWinkelAbsolut(const t_coord &P0, const t_coord &P1)

  - Beschreibung
  --------------
  Berechnet den Winkel Absolut zur Positiven X-Achse.


  - Parameter
  -----------
  E: P0, P1 t_coorde der Linie P0->P1, dessne Winkel berechnet werden soll

  - allgemeine Ein- und Ausgabeeinheiten
  --------------------------------------
  keine

  - Returncode
  ------------
  den Winkel (0-90(I.),90-180(II.),180-270(III.),270-360(IV.))
  zur X-Achse des I.Quadranten in GRAD....

  - Historie
  ----------
  29.6.1994 Sven Goethel

------------------------------------------------------------------------------*/
t_real MetaGeometry::GetWinkelAbsolut(const t_coord &P0, const t_coord &P1)
{
  t_real result;

  // Winkelberechnung .... 0-90 Grad !           
  t_real nn=VektorBetrag(P0, P1);
  if(nn == 0.0 ) return 0.0;
  result=asin(abs(P1.y-P0.y)/nn);
  // vom Bogenmass zum Gradmass
  result=result*180/M_PI;

  // Fallunterscheidung; in welchem Quadranten liegt der Winkel .....
  // P1[0]-P0[0] ... Cosinus Vorzeichen (P1.x-P0.x) !
  // P1[1]-P0[1] ... Sinus Vorzeichen   (P1.y-P0.y) !
  if ( P1.x-P0.x<=0 && P1.y-P0.y>=0 ) result=180-result;
  else if ( P1.x-P0.x<=0 && P1.y-P0.y<=0 ) result=180+result;
  else if ( P1.x-P0.x>=0 && P1.y-P0.y<=0 ) result=360-result;
  // else if ( P1.x-P0.x>0 && P1.y-P0.y>0 ) I.Quadrant keine Aenderung !

  return result;
}


// Berechne den eingeschlossenen Winkel der Strecken P0->P1 und P1->P2 !!!
// Es wird hier der Cosinussatz verwendet !!
// Falls kein Ergebniss moeglich ist, wird ein Wert < 0.0 returniert !!!
t_real MetaGeometry::GetWinkel12 ( const t_coord &P0, const t_coord &P1,
                                   const t_coord &P2
                                 )
{
  double a=(P0-P2).length().DVal();
  double b=(P0-P1).length().DVal();
  double c=(P1-P2).length().DVal();
  if(b<=EPSILON || c<=EPSILON) return -1.0;
  if(b*c==0.0) INT_ERR(__LINE__);

  return acos ( (a*a-b*b-c*c)/(-2*b*c) ) * 180.0 / M_PI;
}

/*------------------------------------------------------------------------------

  ElementFunktion der Klasse MetaGeometry
  --------------------------------------------
  t_real ::GetWinkel12 ( const t_coord P0, const t_coord P1,
               const t_coord P2
             );

  - Beschreibung
  --------------
  Berechnet den Winkel zwischen den Linien P0-P1 und P1-P2 !!
  Ist der Endpunkt der Linie ConA nicht gleich Anfangspunkt der Linie
  von ConB, so wird eine negative Zahl ausgegeben !!!


  - Parameter
  -----------
  E: P0, P1 - Die eine Linie
  E: P1, P2 - Die andere Linie

  - allgemeine Ein- und Ausgabeeinheiten
  --------------------------------------
  keine

  - Returncode
  ------------
  den Winkel (0-90(I.),90-180(II.),180-270(III.),270-360(IV.))
  zur X-Achse des I.Quadranten in GRAD....

  - Historie
  ----------
  29.6.1994 Sven Goethel

------------------------------------------------------------------------------*/
t_real MetaGeometry::GetWinkel12 ( const t_coord &P0, const t_coord &P1,
                                   const t_coord &P2, t_rotation rot
                   )
{
  t_real Alpha=GetWinkelAbsolut(P0, P1);
  t_real Beta=GetWinkelAbsolut(P1, P2);
  return GetWinkel12(Alpha, Beta, rot);
}

t_real MetaGeometry::GetWinkel12 ( t_real Alpha, t_real Beta,
                                   t_rotation rot
                                 )
{
  // Alpha Umdrehen
  Alpha=AddWinkel(Alpha,180);

  // Innerer Winkel Zwischen Alpha und Beta Linksherum berechnen !!!!
  if(Beta>Alpha) Beta=Beta-360; // VORZEICHEN WICHTIG
  Alpha = SubWinkel(Alpha,Beta);
  if(rot==CW) Alpha = SubWinkel(360, Alpha);
  return Alpha;
}

  
t_rotation MetaGeometry::CalcRotation( const t_coord & p1, const t_coord & p2, 
                                       const t_coord & p3 )
{
    t_coord NullCoord,d;                                 
    
    d=p2-p1;
    t_real alpha = GetWinkelAbsolut(NullCoord, d)*M_PI/180.0;

    d=p3-p2;
    t_real theta = GetWinkelAbsolut(NullCoord, d)*M_PI/180.0;

    t_real diff = theta - alpha;
    if ((0.0 < diff.DVal() && diff.DVal() < M_PI) || diff.DVal() < -M_PI) 
      return CCW;
    else   
      return CW;
}

  
bool MetaGeometry::ArcConvertPPP2PMP ( const t_coord & p1, const t_coord & p2, 
                                       const t_coord & p3,             
                                       t_coord & pS, t_coord & pM, 
                                       t_coord & pE, t_rotation &rot
                                     )
{
  if(p1==p2 || p1==p3 || p3==p2) return false;
  rot=CalcRotation(p1, p2, p3);
  pS=p1; pE=p3;
  
  t_coord d12=p1-p2;           
  t_coord d13=p1-p3;           
                    
  if(d12.length()==0.0) INT_ERR(__LINE__);                  
  if(d13.length()==0.0) INT_ERR(__LINE__);                  
  t_real s12 = asin(d12.y / d12.length());
  t_real s13 = asin(d13.y / d13.length());
  if (s12 == s13) return false;

  t_real len1 = p1.x * p1.x + p1.y * p1.y;
  t_real len2 = p2.x * p2.x + p2.y * p2.y;
  t_real len3 = p3.x * p3.x + p3.y * p3.y;
  
  t_real resy = (d12.x * (len3 - len1) - d13.x * (len2 - len1)) /
                (2 * (d13.x * d12.y - d12.x * d13.y));
  t_real resx;              
  if (p1.x != p3.x)
    resx = (len3 + 2.0 * resy * d13.y - len1) / (2.0 * -1.0 * d13.x);
  else
    resx = (len2 + 2.0 * resy * d12.y - len1) / (2.0 * -1.0 * d12.x);
  pM.x = resx;
  pM.y = resy;
  #ifdef _Windows
    ASSERT((pS-pM).length()==(pM-pE).length());
  #else  
    assert((pS-pM).length()==(pM-pE).length());
  #endif  

  return true;
}                                                    
                                  
bool MetaGeometry::ArcConvertPMP2PPP ( const t_coord & pS, const t_coord & pM, 
                                       const t_coord & pE, t_rotation rot,             
                                       t_coord & p1, t_coord & p2, 
                                       t_coord & p3
                                     )               
{ 
  if(pS==pM || pS==pE || pM==pE) return false;
  p1=pS; p3=pE;
                           
  t_coord h=(pE-pS)/2.0;
  t_coord O=pS+h;

  t_real Alpha=GetWinkelAbsolut(pS, O);
  t_real Beta;
  if(rot==CCW)
    Beta=MetaGeometry::SubWinkel(Alpha, 90.0);  
  else  
    Beta=MetaGeometry::AddWinkel(Alpha, 90.0);  
    
  t_coord G(cos(Beta*M_PI/180),sin(Beta*M_PI/180) ,0.0);
  t_coord SchnittPunkte[2];
  
  int snumber=SchnittGrdKreis ( O, G, pM, (pM-pS).length(), SchnittPunkte);
  if(snumber>=1)
  {
    if( TestBogenPunkt (pS, pM, pE, rot, SchnittPunkte[0]) )
      p2=SchnittPunkte[0];  
    if(snumber==2)
    {  
      if( TestBogenPunkt (pS, pM, pE, rot, SchnittPunkte[1]) )
        p2=SchnittPunkte[1];
    }
    else TRACE("ArcConvertPMP2PPP had a funny line for intersection with the circle\n");    
  }  
  else TRACE("ArcConvertPMP2PPP had a very funny line for intersection with the circle\n");    
  
  return true;
}                                                    
                                  

bool MetaGeometry::TestBogenPunkt (const t_coord &ap, const t_coord &mp, 
                                   const t_coord &ep, t_rotation rot, 
                                   const t_coord &p
                                  ) 
{        
  if ( ap==p || ep==p ) return true;
  if ( ap==ep) return false;

  t_real radius = (mp-ap).length();
  t_real radius_p = (mp-p).length();
  if( radius_p != radius ) return false; 
  if( radius == 0.0 ) return radius_p==0.0; 

  t_coord ma=ap-mp;
  t_coord me=ep-mp;
  t_coord mc=p-mp;
  
  /*  Der Winkel vom Vektor <ma> nach Vektor <me> ist durch
      <swab> und <cuadwab> eindeutig bestimmt, wobei:
      <swab>    = (Sinus des Winkels von <ma> nach <me> mal Radius zum Quadrat) &
      <cuadwab> = (Quadrant des Winkels von <ma> nach <me>).
      Die Winkel selbst wollen wir nicht bestimmen. Nur die relative Lage
      der Vektoren <ma>, <me> und <mc> wird herausgefunden und auf <rot_p>
      gespeichert. 
  */
  /* Vorzeichen von <radius> = Vorzeichen von cos(Winkel von <ma> nach <me>).*/
  radius=ScalarProduct(ma,me);
  t_real swab=ma.x*me.y-ma.y*me.x;
  t_real cuadwab=((swab.DVal()>=0.0&&radius.DVal()>=0.0)?1:
                  ((swab.DVal()>=0.0&&radius.DVal()<0.0)?2:
                   ((swab.DVal()<0.0&&radius.DVal()<0.0)?3:4)
                  )
                 );
  radius=ScalarProduct(ma,mc);
  t_real swac=ma.x*mc.y-ma.y*mc.x;
  t_real cuadwac=((swac.DVal()>=0.0&&radius.DVal()>=0.0)?1:
                  ((swac.DVal()>=0.0&&radius.DVal()<0.0)?2:
                   ((swac.DVal()<0.0&&radius.DVal()<0.0)?3:4)
                  )
                 );

  if (swac==swab && cuadwab==cuadwac) return true;
  
  /*  Aus nummerischen Gruenden ist diese ueberpruefung nicht redundant!*/
  t_rotation rot_p;
  if ( cuadwab==cuadwac )
  {
    if ( cuadwab == 1 || cuadwab == 4 ) /* Der Sinus ist im 1. und 4. Quadrant monoton wachsend. */
      rot_p = (swac< swab) ? CCW : CW;
    else
      rot_p = (swac< swab) ? CW : CCW;
  }
  else
    rot_p = (cuadwac<cuadwab) ? CCW : CW;

  return rot_p==rot;
}      
                                         
// Point: P, Line: A + s * B ( s = any scalar )                                         
t_real MetaGeometry::DistancePointLine(const t_coord &P, 
                                       const t_coord &A, const t_coord &B
                                      )
{
  return (B * (P-A)).length()/B.length();  
}                                              

bool MetaGeometry::TestStreckenPunkt (const t_coord &pS, const t_coord &pE, 
                                      const t_coord &p
                                     ) 
{                                  
  return DistancePointLine(p, pS, pE-pS)==0.0 &&
         (pS-pE).length() == (pS-p).length() + (pE-p).length() ;
}      
                                  
// Schnittpunkte Gerade Gerade
//
// s = (oa1*gb2-ob1*gb2-gb1*oa2+gb1*ob2)/(-ga1*gb2+ga2*gb1);
// t = -1/(-ga1*gb2+ga2*gb1)*(ga1*oa2-ga1*ob2-ga2*oa1+ga2*ob1);
//
// Ist f2[3] == f3[3], so existiert ein Schnittpunkt.
//
// f2[3] = oa3+s*ga3;
// f3[3] = ob3+t*gb3;
//
// s oder t kann dann in einer der beiden geradengl. eingesetzt werden !!!

bool MetaGeometry::SchnittStreckeStrecke(const t_coord &PA0, const t_coord &PA1, 
                                         const t_coord &PB0, const t_coord &PB1,
                                         t_coord &SchnittPunkt, 
                                         bool &AufStreckeA,
                                         bool &AufStreckeB
                                        )
{ 
  // Die Richtungsvektoren                   :
  t_coord Ga=PA1-PA0;
  t_coord Gb=PB1-PB0;                   
  bool result=SchnittGrdGrd(PA0, Ga, PB0, Gb, SchnittPunkt);
  
  if( result )
  {                        
    if( TestStreckenPunkt( PA0, PA1, SchnittPunkt) )
      AufStreckeA=true;    
    else  
      AufStreckeA=false;    
      
    if( TestStreckenPunkt( PB0, PB1, SchnittPunkt) )
      AufStreckeB=true;    
    else  
      AufStreckeB=false;    
  }
  return result;
}                 
                   
                   
bool MetaGeometry::SchnittGrdGrd(const t_coord &Oa, const t_coord &Ga, 
                                 const t_coord &Ob, const t_coord &Gb,
                                 t_coord &SchnittPunkt
                                )
{          
  double oa1=Oa.x.DVal();
  double oa2=Oa.y.DVal();
  double oa3=Oa.z.DVal();
  double ga1=Ga.x.DVal();
  double ga2=Ga.y.DVal();
  double ga3=Ga.z.DVal();
  
  double ob1=Ob.x.DVal();
  double ob2=Ob.y.DVal();
  double ob3=Ob.z.DVal();
  double gb1=Gb.x.DVal();
  double gb2=Gb.y.DVal();
  double gb3=Gb.z.DVal();
  
  double s_nenner = -ga1*gb2+ga2*gb1;
  if(s_nenner==0) return false;                        
  double s = (oa1*gb2-ob1*gb2-gb1*oa2+gb1*ob2)/s_nenner;
  
  double t_nenner = (-ga1*gb2+ga2*gb1)*(ga1*oa2-ga1*ob2-ga2*oa1+ga2*ob1);
  if(t_nenner==0) return false;                        
  double t = -1/t_nenner;
  
  if( fabs ( ( oa3+s*ga3 ) - ( ob3+t*gb3 ) )<= EPSILON )
  {
    SchnittPunkt.x=oa1+s*ga1;
    SchnittPunkt.y=oa2+s*ga2;
    SchnittPunkt.z=oa3+s*ga3;
    return true;
  }
  return false;
}                  



// Schnittpunkte Gerade - Bogen
//
// O = [ o1,o2,o3 ] ; Der Ortsvektor der Geradengleichung
// G = [ g1,g2,g3 ] ; Der Richtungsvektor der Geradengleichung
// M = [ m1,m2,m3 ] ; Der Kreismittelpunkt 
// r                ; Der Radius
// s1, s2           ; Die evt. 2 faktoren fuer die Geradengleichung
//                    ( Schnittpunkte ) 
// return           ; ( 0, 1, 2 ) Die Schnittpunkteanzahl.
//
// Die Ortsvektoren koennen wie folgt berechnet werden :
/*
      X[2] = o2+s*g2;
      X[1] = o1+s*g1;
      X[3] = o3+s*g3;
*/

int MetaGeometry::SchnittStreckeBogen  (const t_coord &pA0, const t_coord &pA1, 
                                        const t_coord &pBS, const t_coord &pBM,
                                        const t_coord &pBE, t_rotation rot,
                                        t_coord SchnittPunkte[], 
                                        bool AufStrecke[], bool AufBogen[]
                                       )
{
  // Die Richtungsvektoren                   :
  t_coord g=pA1-pA0;
  int snumber=SchnittGrdKreis(pA0, g, pBM, (pBM-pBS).length(), SchnittPunkte);
      
  for(int i=0; i<snumber; i++)    
  {
    if( TestStreckenPunkt( pA0, pA1, SchnittPunkte[i]) )
      AufStrecke[i]=true;    
    else  
      AufStrecke[i]=false;    
      
    if( TestBogenPunkt (pBS, pBM, pBE, rot, SchnittPunkte[i]) )
      AufBogen[i]=true;    
    else  
      AufBogen[i]=false;    
  }
  return snumber;
}                                       
  
int MetaGeometry::SchnittGrdKreis      (const t_coord &O, const t_coord &G, 
                                        const t_coord &M, const t_real &radius,
                                        t_coord SchnittPunkte[]
                                       )
{                               
  // Argument transfer !!!
  double g1=G.x.DVal();
  double g2=G.y.DVal();
  double g3=G.z.DVal();
  double m1=M.x.DVal();
  double m2=M.y.DVal();
  double m3=M.z.DVal();
  double o1=O.x.DVal();
  double o2=O.y.DVal();
  double o3=O.z.DVal();
  double r=radius.DVal();
  double s1, s2;
  int snumber=0;
  
  // maple's optimized stuff !!!
  double t1 = g1*g1;
  double t2 = g2*g2;
  double t3 = g3*g3;
  double t6 = o1*g1;
  double t7 = g1*m1;
  double t8 = o2*g2;
  double t9 = g2*m2;
  double t10 = o3*g3;
  double t11 = g3*m3;

  double t12 = r*r;
  double t18 = o2*o2;
  double t32 = o3*o3;
  double t34 = o1*o1;
  double t37 = m3*m3;
  double t39 = t1*t12+2.0*t1*o3*m3+2.0*t2*o1*m1-t1*t18+2.0*t3*o1*m1
               +2.0*t3*o2*m2+2.0*t9*t11+2.0*t2*o3*m3+2.0*t1*o2*m2
               +2.0*t7*t11-2.0*t8*t11-2.0*t7*t10-t1*t32-t2*t34-t2*t32
               -t2*t37;
  double t42 = m1*m1;
  double t45 = m2*m2;
  double t59 = t2*t12-t3*t34-t3*t42-t3*t18-t3*t45+t3*t12-t1*t37+2.0*t6*t8
               -t2*t42-t1*t45-2.0*t6*t9+2.0*t6*t10-2.0*t6*t11-2.0*t7*t8
               +2.0*t7*t9+2.0*t8*t10-2.0*t9*t10+t39;

  if(t59<0) 
    snumber=0;
  else
  {
    double t61 = sqrt(t59);
    if(t1+t2+t3==0.0) INT_ERR(__LINE__);
    s1 = 1/(t1+t2+t3)
         *(-2.0*t6+2.0*t7-2.0*t8+2.0*t9-2.0*t10+2.0*t11+2.0*t61)/2;

    if(t59==0) 
      snumber=1;
    else
    {  
      s2 = 1/(t1+t2+t3)
           *(-2.0*t6+2.0*t7-2.0*t8+2.0*t9-2.0*t10+2.0*t11-2.0*t61)/2;
  
      snumber=2;
    }  
  }
  if(snumber>=1)
  {
      SchnittPunkte[0].x = o1+s1*g1;
      SchnittPunkte[0].y = o2+s1*g2;
      SchnittPunkte[0].z = o3+s1*g3;
  }
  if(snumber==2)
  {
      SchnittPunkte[1].x = o1+s2*g1;
      SchnittPunkte[1].y = o2+s2*g2;
      SchnittPunkte[1].z = o3+s2*g3;
  }
  return snumber;
}


int MetaGeometry::SchnittBogenBogen    (
                                        const t_coord &pB1S, const t_coord &pB1M,
                                        const t_coord &pB1E, t_rotation B1rot,
                                        const t_coord &pB2S, const t_coord &pB2M,
                                        const t_coord &pB2E, t_rotation B2rot,
                                        t_coord SchnittPunkte[], 
                                        bool AufBogen1[], bool AufBogen2[]
                                       )
{                               
  int snumber=SchnittKreisKreis(pB1M, (pB1M-pB1S).length(), 
                                pB2M, (pB2M-pB2S).length(), 
                                SchnittPunkte
                               ); 
  for(int i=0; i<snumber; i++)    
  {
    if( TestBogenPunkt (pB1S, pB1M, pB1E, B1rot, SchnittPunkte[i]) ) 
      AufBogen1[i]=true;    
    else  
      AufBogen1[i]=false;    
      
    if( TestBogenPunkt (pB2S, pB2M, pB2E, B2rot, SchnittPunkte[i]) )
      AufBogen2[i]=true;    
    else  
      AufBogen2[i]=false;    
  }
  return snumber;
}

int MetaGeometry::SchnittKreisKreis    (const t_coord &M1, const t_real &r1,
                                        const t_coord &M2, const t_real &r2, 
                                        t_coord SchnittPunkte[]
                                       )
{                               
  /* R1 : vektor M1 -> P1 */
  long double r1xr1=(long double)(r1.DVal())*(long double)(r1.DVal());
  
  /* R2 : vektor M2 -> P2 */
  long double r2xr2=(long double)(r2.DVal())*(long double)(r2.DVal());
  
  /* MM : vektor M2 -> M1 */
  t_coord MM=M1-M2;
  long double mm=MM.length().DVal();
  long double mmxmm=mm*mm;
  
  if ( mmxmm > r1xr1 + r2xr2 + 2.0 * r1.DVal() * r2.DVal() ||
       r1xr1 > mmxmm + r2xr2 + 2.0 * mm * r2.DVal() ||
       r2xr2 > r1xr1 + mmxmm + 2.0 * r1.DVal() * mm 
     )
  {                      
    /* the circles are not in the same area, or one circle includes the other */
    return 0;
  }  

  if ( mm==0.0 )
  {                         
    /* the middle points are even */
    if (fabsl(r1xr1-r2xr2)==0.0)
    {
      /* the difference of the radiant-squares is too big ??? */
      return 0;
    }  

    /* the circles are even */
    return 3;
  }  
  
  /* we are going to create the line o+s*g in a way ( s=1 const), */
  /* that the line intersect the circle intersection points */
  t_coord o, g; 
  
  t_coord PosZAchse(0.0,0.0,1.0);
  g=MM*PosZAchse; /* g is a lot to M2-M1 -> Arc(MM, g)==90.0 degree */
  long double vorzeichen = (mmxmm+r2xr2<r1xr1)?-0.5:0.5;
  o=MM;
  o*=vorzeichen + (vorzeichen*(r2xr2-r1xr1)/mmxmm);
  o+=M2;
  
  return SchnittGrdKreis (o, g, M1, r1, SchnittPunkte);
}
