// author : Sven Goethel - Stapenhorststrasse 35a - 33615 Bielefeld
//

#ifndef _FXMTYPES_H

  #ifndef _METATYPES_H
    #define _METATYPES_H 

    #include <math.h>
    #include "strclass.h"
    #include "bool.h"

    #define EPSILON    (0.1/1000)

    typedef StrClass  t_string;

    typedef bool t_bool;

    typedef enum { L, A } t_contour;

    typedef enum { CW, CCW } t_rotation;

    typedef char  t_char;

    typedef int  t_int;

    typedef long  t_long;

    #ifndef M_PI
      #define M_PI 3.141592653
    #endif

    // a very very transparent thing, now ....
    class t_real
    {
      public:
        t_real():r(0.0) {}
        t_real (const double & val){ r = val; }
        t_real (const t_real & val){ r = val.r; }

        t_real& operator+=(const t_real &val){ r += val.r; return(*this); }
        t_real& operator-=(const t_real &val){ r -= val.r; return(*this); }
        t_real& operator*=(const t_real &val){ r *= val.r; return(*this); }
        t_real& operator/=(const t_real &val)
        { if(val.r==0) INT_ERR(__LINE__); r /= val.r; return(*this); }

        int operator==(const t_real &val) const 
        { return(fabs(r-val.r)<EPSILON); }
        int operator!=(const t_real &val) const 
        { return(!((*this)==val)); }
        int operator>(const t_real &val) const 
        { return(r>val.r); }
        int operator>=(const t_real &val) const 
        { return(r>=val.r); }
        int operator<(const t_real &val) const 
        { return(r<val.r); }
        int operator<=(const t_real &val) const 
        { return(r<=val.r); }

        double  value() const { return r; }
        double  DVal()  const { return r; }
        int     IVal()  const { int ret=(int)(r+0.5); return ret; }

      private:
        double r;
    };

    inline t_real operator-(const t_real &a, const t_real &b)
    { return a.value() - b.value() ; }
    inline t_real operator+(const t_real &a, const t_real &b)
    { return a.value() + b.value() ; }
    inline t_real operator*(const t_real &a, const t_real &b)
    { return a.value() * b.value() ; }
    inline t_real operator/(const t_real &a, const t_real &b)
    { if(b.value()==0) INT_ERR(__LINE__); return a.value() / b.value() ; }

    inline t_real sin(const t_real &val)  { return sin(val.value()); }
    inline t_real asin(const t_real &val) { return asin(val.value()); }
    inline t_real cos(const t_real &val)  { return cos(val.value()); }
    inline t_real acos(const t_real &val) { return acos(val.value()); }
    inline t_real tan(const t_real &val)  { return tan(val.value()); }
    inline t_real atan(const t_real &val) { return atan(val.value()); }
    inline t_real abs(const t_real &val)  { return fabs(val.value()); }
    inline t_real pow(const t_real &val, const t_real &exp) 
    { return pow(val.value(), exp.value()); }
    inline t_real sqrt(const t_real &val)
    { return sqrt(val.value()); }
    inline t_real sqr(const t_real &val)
    { return pow(val.value(), 2.0); }

    struct t_coord
    {
      t_coord():x(0.0),y(0.0),z(0.0){}
      t_coord(const t_real &xx, const t_real &yy, const t_real &zz)
      :x(xx),y(yy),z(zz){}

      int operator==(const t_coord &a) const
      { return x==a.x && y==a.y && z==a.z; }
      int operator!=(const t_coord &a) const
      { return !((*this)==a); }

      int operator>(const t_coord &a) const
      { return x>a.x && y>a.y && z>a.z; }
      int operator>=(const t_coord &a) const
      { return x>=a.x && y>=a.y && z>=a.z; }

      int operator<(const t_coord &a) const
      { return x<a.x && y<a.y && z<a.z; }
      int operator<=(const t_coord &a) const
      { return x<=a.x && y<=a.y && z<=a.z; }

      t_coord& operator+=(const t_coord &a)
      { x+=a.x; y+=a.y; z+=a.z; return *this; }
      t_coord& operator-=(const t_coord &a)
      { x-=a.x; y-=a.y; z-=a.z; return *this; }

      t_coord& operator/=(const t_real &a)
      { if(a.value()==0) INT_ERR(__LINE__); x/=a; y/=a; z/=a; return *this; }
      t_coord& operator*=(const t_real &a)
      { x*=a; y*=a; z*=a; return *this; }

      t_real length() const { return sqrt(x*x+y*y+z*z); }

      t_real  x;
      t_real  y;
      t_real  z;
    };

    inline t_coord operator+(const t_coord &ak, const t_coord &bk)
    { t_coord r=ak; r+=bk; return r; }
    
    inline t_coord operator-(const t_coord &ak, const t_coord &bk)
    { t_coord r=ak; r-=bk; return r; }
    
    inline t_coord operator*(const t_coord &ak, const t_real &b)
    { t_coord r=ak; r*=b; return r; }
    
    inline t_coord operator/(const t_coord &ak, const t_real &b)
    { t_coord r=ak; if(b.value()==0) INT_ERR(__LINE__); r/=b; return r; }
    
    inline t_coord operator*(const t_real &b, const t_coord &ak)
    { t_coord r=ak; r*=b; return r; }
    
    inline t_coord operator*(const t_coord &a, const t_coord &b)
    { t_coord axb(a.y*b.z - a.z*b.y, a.z*b.x - a.x*b.z, a.x*b.y - a.y*b.x);
      return axb;
    }
    
    inline t_real ScalarProduct(const t_coord &a, const t_coord &b)
    { return a.x*b.x+a.y*b.y+a.z*b.z;
    }
    /*
      // for future ANSI-C++ features the scalar-product !!!!!!!!
      inline t_real operator*(const t_coord &a, const t_coord &b)
      { return a.x*b.x+a.y*b.y+a.z*b.z;
      }
    */  
    
    struct t_orient
    {
      t_orient():a(0.0),b(0.0),c(0.0){}
      t_orient(const t_real &aa, const t_real &bb, const t_real &cc)
      :a(aa),b(bb),c(cc){}

      int operator==(const t_orient &_a) const
      { return a==_a.a && b==_a.b && c==_a.c; }

      int operator!=(const t_orient &a) const
      { return !((*this)==a); }

      t_real  a;
      t_real  b;
      t_real  c;
    };

  #endif
#endif


